package com.github.jfcloud.gene.lis.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import cn.hutool.db.Entity;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.github.jfcloud.gene.lis.config.LisDb;
import com.github.jfcloud.gene.lis.entity.HtScienceApply;
import com.github.jfcloud.gene.lis.entity.HtScienceApplyItem;
import com.github.jfcloud.gene.sample.entity.SampleCheckOrder;
import com.github.jfcloud.gene.sample.entity.SampleInfo;
import com.github.jfcloud.gene.sample.enums.ScientificSampleTypeEnum;
import com.github.jfcloud.gene.sample.mapper.SampleCheckOrderMapper;
import com.github.jfcloud.gene.sample.mapper.SampleInfoMapper;
import com.github.jfcloud.gene.sample.service.SampleAnimalService;
import com.github.jfcloud.gene.sample.service.SampleCheckItemSerumService;
import com.github.jfcloud.gene.sample.vo.CheckItemSerumVo;
import com.github.jfcloud.gene.sample.vo.SampleAnimalVo;
import com.github.jfcloud.gene.sample.vo.SampleCheckSerumVo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class LisSyncService {

    private final LisDb lisDb;
    private final SampleInfoMapper sampleInfoMapper;
    private final SampleCheckOrderMapper checkOrderMapper;
    private final SampleAnimalService sampleAnimalService;
    private final SampleCheckItemSerumService sampleCheckItemSerumService;

    /**
     * 同步LIS
     */
    public void sync(Long sampleId) {
        SampleAnimalVo animalDetail = sampleAnimalService.getDetail(sampleId);
        Assert.notNull(animalDetail, "同步LIS失败，动物样本检测不存在");
        Assert.isTrue(animalDetail.getScientificSampleType().contains(ScientificSampleTypeEnum.serum.getDescription()), "同步LIS失败，只有包含血清样本送检才可以同步");
        SampleCheckSerumVo serum = animalDetail.getOrder().getSerum();

        SampleCheckOrder checkOrder = checkOrderMapper.selectById(serum.getId());
        if (StrUtil.isNotBlank(checkOrder.getExtData())) {
            JSONObject extObj = JSON.parseObject(checkOrder.getExtData());
            Assert.isFalse(extObj.containsKey("lisId"), "同步LIS失败，已同步过");
        }

        SampleInfo sampleInfo = sampleInfoMapper.selectById(sampleId);

        //HT_SCIENCE_APPLY主键非自增，为年份2位+月份2位+日期2位+483+7位序列号，为了避免冲突，这里使用480
        String idPrefix = sampleInfo.getApplyTime().format(DateTimeFormatter.ofPattern("yyMMdd")) + "480";
        long firstId = Long.parseLong(String.format("%s%07d", idPrefix, 1));
        //查询当前最大的id
        String sql = String.format("select top 1 id from dbo.HT_SCIENCE_APPLY where id like '%s%%' order by id desc ",  idPrefix);
        List<Entity> idEntities = lisDb.query(sql);
        if (CollUtil.isNotEmpty(idEntities)) {
            firstId = idEntities.get(0).getLong("id") + 1;
        }

        //todo 同步LIS
        List<HtScienceApply> applyList = new ArrayList<>();
        List<HtScienceApplyItem> applyItemList = new ArrayList<>();
        for (CheckItemSerumVo item : serum.getItems()) {
            HtScienceApply scienceApply = new HtScienceApply();
            scienceApply.setId(firstId++);
            scienceApply.setPhone(serum.getApplyUserMobile());
            scienceApply.setEmail(serum.getApplyUserEmail());
            applyList.add(scienceApply);
        }

        lisDb.execute(db -> {
            try {
                List<Entity> applyEntities = applyList.stream().map(x -> Entity.create("dbo.HT_SCIENCE_APPLY").parseBean(x)).collect(Collectors.toList());
                db.insert(applyEntities);
                List<Entity> applyItemEntities = applyItemList.stream().map(x -> Entity.create("dbo.HT_SCIENCE_APPLY_ITEM").parseBean(x)).collect(Collectors.toList());
                db.insert(applyItemEntities);
            } catch (Exception e) {
                log.error("同步LIS失败", e);
            }
        });
    }

    /**
     * 取消同步LIS
     */
    public void cancelSyncLis(Long sampleId) {
        log.info("取消同步LIS");
    }
}
